# SPDX-FileCopyrightText: 2024-present OLIST TINY TECNOLOGIA LTDA
#
# SPDX-License-Identifier: MIT
from django.apps import AppConfig


class HealthyApp(AppConfig):
    name = "healthy"
    verbose_name = "Django Healthy"
